//
//  BFVideoThumbProgressView.swift
//  BFRecordScreenKit
//
//  Created by 胡志强 on 2021/12/3.
//

import Foundation
import UIKit
import AVFoundation
import BFCommonKit
import SnapKit

class BFVideoThumbProgressView: UIView {
    var videoAsset : AVURLAsset?
    var dragScrollProgressHandle : ((Float) -> Void)?
    var dragEndHandle : ((Float) -> Void)?
    var isDrag = false
    var progress:Double = 0 {
        didSet{
            if let second = self.videoAsset?.duration.seconds, second > 0 {
                let w = progressView.contentSize.width - width
                progressView.contentOffset = CGPoint(x: progress * w / second, y: 0)
            }
        }
    }
    
    var thumbImgs = [UIImage]()
    
    init(frame: CGRect, videoAsset:AVURLAsset) {
        super.init(frame: frame)
        self.videoAsset = videoAsset
        addSubview(progressView)
        
        let line = UIView()
        line.backgroundColor = .white
        line.layer.shadowColor = UIColor.black.cgColor
        line.layer.shadowOffset = CGSize(width: 1, height: 1)
        addSubview(line)
        line.snp.makeConstraints { make in
            make.width.equalTo(2)
            make.center.height.equalToSuperview()
        }
        
        splitVideoFileUrlFps(urlAsset: videoAsset, fps: 2) {[weak self] isSuccess, images in
            if isSuccess{
                self?.thumbImgs = images!
                DispatchQueue.main.async {[weak self] in
                    if let sself = self{
                        for (i, img) in images!.enumerated() {
                            let iv = UIImageView(image: img)
                            iv.contentMode = .scaleAspectFill
                            iv.clipsToBounds = true
                            sself.progressView.addSubview(iv)
                            iv.snp.makeConstraints { make in
                                make.left.equalTo(CGFloat(i) * sself.height + sself.width * 0.5)
                                make.top.height.equalToSuperview()
                                make.width.equalTo(sself.height)
                            }
                        }
                        sself.progressView.contentSize = CGSize(width: CGFloat(images!.count) *  sself.height + sself.width, height: sself.height)
                    }
                }
            }
        }
    }

    required init?(coder: NSCoder) {
        fatalError("init(coder:) has not been implemented")
    }
    
    lazy var progressView : UIScrollView = {
        let sv = UIScrollView()
        sv.bounces = false
        sv.backgroundColor = .clear
        sv.decelerationRate = .fast
        sv.showsHorizontalScrollIndicator = false
        sv.delegate = self
        
        return sv
    }()
    
    override func layoutSubviews() {
        super.layoutSubviews()
        progressView.snp.makeConstraints { make in
            make.edges.equalToSuperview()
        }
        
    }
}

extension BFVideoThumbProgressView : UIScrollViewDelegate {
    func scrollViewDidScroll(_ scrollView: UIScrollView) {
        if isDrag{
            let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - self.width)
            self.dragScrollProgressHandle?(Float(dur))
        }
    }
    
    func scrollViewWillBeginDragging(_ scrollView: UIScrollView) {
        isDrag = true
        let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - self.width)
        self.dragScrollProgressHandle?(Float(dur))
    }
    
    func scrollViewDidEndDragging(_ scrollView: UIScrollView, willDecelerate decelerate: Bool) {
        if !decelerate {
            let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - self.width)
            isDrag = false
            dragEndHandle?(Float(dur))
        }
    }
    
    func scrollViewDidEndDecelerating(_ scrollView: UIScrollView) {
        let dur = scrollView.contentOffset.x / (scrollView.contentSize.width - self.width)
        isDrag = false
        dragEndHandle?(Float(dur))

    }
    
}
